package de.unikoblenz.fgbks.core.dmn.domain.vdmn;

import de.unikoblenz.fgbks.base.utils.boundary.Boundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.BooleanBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.DateBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.DoubleBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.IntegerBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.InvalidBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.LongBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;
import java.time.LocalDateTime;
import java.util.Optional;

public enum VTypeRef {
  DATE("date", LocalDateTime.class, DateBoundary.class),
  STRING("string", String.class, StringBoundary.class),
  INTEGER("integer", Integer.class, IntegerBoundary.class),
  BOOLEAN("boolean", Boolean.class, BooleanBoundary.class),
  DOUBLE("double", Double.class, DoubleBoundary.class),
  LONG("long", Long.class, LongBoundary.class);

  private String name;
  private Class<?> javaClass;
  private Class<? extends Boundary> boundaryClass;

  public String getName() {
    return name;
  }

  public Class<?> getJavaClass() {
    return javaClass;
  }

  public static VTypeRef getTypeRefFromName(String name) {
    for (VTypeRef typeRef : values()) {
      if (typeRef.name.equals(name)) {
        return typeRef;
      }
    }
    return null;
  }

  public static VTypeRef getTypeRefFromJavaClass(Class<?> javaClass) {
    for (VTypeRef typeRef : values()) {
      if (typeRef.javaClass.equals(javaClass)) {
        return typeRef;
      }
    }
    return null;
  }

  public Optional<Boundary> getBoundaryFromText(String text) {
    try {
      return Optional.of(boundaryClass.getDeclaredConstructor(String.class).newInstance(text));
    } catch (Exception ignored) {
      return Optional.of(InvalidBoundary.getInstance());
    }
  }

  VTypeRef(String name, Class<?> javaClass, Class<? extends Boundary> boundaryClass) {
    this.name = name;
    this.javaClass = javaClass;
    this.boundaryClass = boundaryClass;
  }
}
