package de.unikoblenz.fgbks.core.dmn.domain.vdmn.parser;

import de.unikoblenz.fgbks.base.builder.AbstractBuilder;
import de.unikoblenz.fgbks.core.dmn.domain.common.RowNumber;
import de.unikoblenz.fgbks.core.dmn.domain.ids.DecisionId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.DecisionTableId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.DefinitionId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.InputEntryId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.InputExpressionId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.InputId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.OutputEntryId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.OutputId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.RuleId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDefinition;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl.VDmnDecisionImpl;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl.VDmnDecisionTableImpl;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl.VDmnDefinitionImpl;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl.VDmnInputColumnImpl;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl.VDmnInputValueImpl;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl.VDmnOutputColumnImpl;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl.VDmnOutputValueImpl;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl.VDmnRuleImpl;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.enterprise.context.ApplicationScoped;
import org.camunda.bpm.model.dmn.DmnModelInstance;
import org.camunda.bpm.model.dmn.instance.Decision;
import org.camunda.bpm.model.dmn.instance.DecisionTable;
import org.camunda.bpm.model.dmn.instance.Input;
import org.camunda.bpm.model.dmn.instance.InputEntry;
import org.camunda.bpm.model.dmn.instance.Output;
import org.camunda.bpm.model.dmn.instance.OutputEntry;
import org.camunda.bpm.model.dmn.instance.Rule;

@ApplicationScoped
public class SimpleVDmnParser implements DmnModelInstanceWrapper {

  @Override
  public VDmnDefinition getVDmnDefinition(DmnModelInstance dmnModel) {
    // Get the definition
    VDmnDefinitionImpl.Builder definitionBuilder =
        VDmnDefinitionImpl.getBuilder()
            .withDefinitionId(new DefinitionId(dmnModel.getDefinitions().getId()));
    // Iterate for all decisions
    for (Decision decision : dmnModel.getModelElementsByType(Decision.class)) {
      // Create Builder for DmnDecision
      VDmnDecisionImpl.Builder decisionBuilder =
          VDmnDecisionImpl.getBuilder().withDecisionId(new DecisionId(decision.getId()));
      // Add DmnDecisionBuilder to definitionBuilder
      definitionBuilder.addDmnDecisionFromBuilder(decisionBuilder);
      // Get the decisionTable
      DecisionTable decisionTable =
          new ArrayList<>(decision.getChildElementsByType(DecisionTable.class)).get(0);
      // Create Builder for DmnDecisionTable
      VDmnDecisionTableImpl.Builder decisionTableBuilder =
          VDmnDecisionTableImpl.getBuilder()
              .withDecisionTableId(new DecisionTableId(decisionTable.getId()))
              .withHitPolicy(decisionTable.getHitPolicy());
      // Add DmnDecisionTableBuilder to decisionBuilder
      decisionBuilder.withDmnDecisionTableFromBuilder(decisionTableBuilder);

      // Create input columns
      List<VDmnInputColumnImpl.Builder> inputColumnBuilders =
          new ArrayList<>(decisionTable.getInputs().size());
      for (Input inputCol : decisionTable.getInputs()) {
        inputColumnBuilders.add(
            VDmnInputColumnImpl.getBuilder()
                .withInputId(new InputId(inputCol.getId()))
                .withInputExpressionId(new InputExpressionId(inputCol.getInputExpression().getId()))
                .withTypeRef(
                    VTypeRef.getTypeRefFromName(inputCol.getInputExpression().getTypeRef())));
        decisionTableBuilder.addDmnInputColumnFromBuilder(
            inputColumnBuilders.get(inputColumnBuilders.size() - 1));
      }

      // Create output columns
      List<VDmnOutputColumnImpl.Builder> outputColumnBuilders =
          new ArrayList<>(decisionTable.getOutputs().size());
      for (Output outputCol : decisionTable.getOutputs()) {
        outputColumnBuilders.add(
            VDmnOutputColumnImpl.getBuilder()
                .withOutputId(new OutputId(outputCol.getId()))
                .withTypeRef(VTypeRef.getTypeRefFromName(outputCol.getTypeRef())));
        decisionTableBuilder.addDmnOutputColumnFromBuilder(
            outputColumnBuilders.get(outputColumnBuilders.size() - 1));
      }

      // Create rows
      Long rowCount = 0L;
      for (Rule rule : decisionTable.getRules()) {
        // Create Builder for rule
        VDmnRuleImpl.Builder dmnRuleBuilder =
            VDmnRuleImpl.getBuilder()
                .withRuleId(new RuleId(rule.getId()))
                .withRowNumber(new RowNumber(++rowCount));
        // Add row to Decision
        decisionTableBuilder.addRuleFromBuilder(dmnRuleBuilder);

        // Input Entries
        Collection<InputEntry> inputElements = rule.getChildElementsByType(InputEntry.class);
        int x;
        x = 0;
        for (InputEntry inputEntry : inputElements) {
          // create Input Value
          VDmnInputValueImpl.Builder vInputValueBuilder =
              VDmnInputValueImpl.getBuilder()
                  .withInputEntryId(new InputEntryId(inputEntry.getId()))
                  .withDescription(
                      inputEntry.getDescription() == null
                          ? ""
                          : inputEntry.getDescription().getTextContent())
                  .withText(inputEntry.getText().getTextContent());
          // Add Input value to rule
          dmnRuleBuilder.addInputValueFromBuilder(vInputValueBuilder);
          // Add Input value to input column
          inputColumnBuilders.get(x++).addValueFromBuilder(vInputValueBuilder);
          // build Value
          vInputValueBuilder.build();
        }

        // Output Entries
        Collection<OutputEntry> outputElements = rule.getChildElementsByType(OutputEntry.class);
        x = 0;
        for (OutputEntry outputEntry : outputElements) {
          // create Output Value
          VDmnOutputValueImpl.Builder vOutputValueBuilder =
              VDmnOutputValueImpl.getBuilder()
                  .withOutputEntryId(new OutputEntryId(outputEntry.getId()))
                  .withDescription(
                      outputEntry.getDescription() == null
                          ? ""
                          : outputEntry.getDescription().getTextContent())
                  .withText(outputEntry.getText().getTextContent());
          // Add Output value to rule
          dmnRuleBuilder.addOutputValueFromBuilder(vOutputValueBuilder);
          // Add Output value to input column
          outputColumnBuilders.get(x++).addValueFromBuilder(vOutputValueBuilder);
          // build Value
          vOutputValueBuilder.build();
        }
        // build rule
        dmnRuleBuilder.build();
      }
      // build cols
      inputColumnBuilders.forEach(AbstractBuilder::build);
      outputColumnBuilders.forEach(AbstractBuilder::build);

      // Build decisionTable
      decisionTableBuilder.build();
      // Build decisionTable
      decisionBuilder.build();
    }
    return definitionBuilder.build();
  }
}
