package de.unikoblenz.fgbks.core.dmn.verification.result;

import de.unikoblenz.fgbks.base.builder.DefaultBuilder;
import de.unikoblenz.fgbks.base.domain.Message;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import javax.json.bind.annotation.JsonbProperty;
import org.apache.commons.lang3.Validate;

public class VerificationResultEntry extends AbstractResultObject {

  public enum VerificationClassification {
    INFO,
    WARNING,
    ERROR,
    FATAL_ERROR;
  }

  private Set<VerificationResultEntryElement> verificationResultEntryElements;
  private Message message;
  private VerificationClassification verificationClassification;

  protected VerificationResultEntry() {
    this.verificationResultEntryElements = ConcurrentHashMap.newKeySet();
  }

  @JsonbProperty("size")
  private int getAmountOfElements() {
    return verificationResultEntryElements.size();
  }

  @JsonbProperty("elements")
  public Set<VerificationResultEntryElement> getVerificationResultEntryElements() {
    return new HashSet<>(verificationResultEntryElements);
  }

  @JsonbProperty("verificationClassification")
  public VerificationClassification getVerificationClassification() {
    return verificationClassification;
  }

  @JsonbProperty("message")
  public Message getMessage() {
    return new Message(message);
  }

  private void addVerificationResultEntry(
      VerificationResultEntryElement verificationResultEntryElement) {
    this.verificationResultEntryElements.add(verificationResultEntryElement);
  }

  public static Builder getBuilder() {
    return new VerificationResultEntry().new Builder();
  }

  public class Builder extends DefaultBuilder<VerificationResultEntry> {

    public Builder addVerificationResultEntryElement(
        VerificationResultEntryElement verificationResultEntryElement) {
      value.addVerificationResultEntry(Validate.notNull(verificationResultEntryElement));
      return this;
    }

    public Builder withMessage(Message message) {
      value.message = new Message(message);
      return this;
    }

    public Builder withClassification(VerificationClassification verificationClassification) {
      value.verificationClassification = verificationClassification;
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.message);
      Validate.notNull(value.verificationClassification);
      Validate.notEmpty(value.verificationResultEntryElements);
    }
  }
}
