package de.unikoblenz.fgbks.core.dmn.verification.result;

import de.unikoblenz.fgbks.core.dmn.domain.ids.AbstractId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.JsonIdentifier;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnValue;
import java.util.HashMap;
import java.util.Map;
import javax.json.bind.annotation.JsonbProperty;
import org.apache.commons.lang3.Validate;

// @JsonbTypeSerializer(VerificationResultEntryElementJsonSerializer.class)
public class VerificationResultEntryElement extends AbstractResultObject {

  private final Map<String, AbstractId> identifier;

  @JsonbProperty("identifier")
  public Map<String, AbstractId> getIdentifier() {
    return new HashMap<>(identifier);
  }

  private VerificationResultEntryElement() {
    identifier = new HashMap<>();
  }

  public static VerificationResultEntryElement create() {
    return new VerificationResultEntryElement();
  }

  public static VerificationResultEntryElement create(VDmnValue vDmnValue) {
    return new VerificationResultEntryElement()
        .withIdentifier(vDmnValue.getDefinitionId())
        .withIdentifier(vDmnValue.getDecisionId())
        .withIdentifier(vDmnValue.getDecisionTableId())
        .withIdentifier(vDmnValue.getDmnColumn().getColumnId())
        .withIdentifier(vDmnValue.getRuleId())
        .withIdentifier(vDmnValue.getId());
  }

  public static VerificationResultEntryElement create(VDmnDecisionTable dmnDecisionTable) {
    return new VerificationResultEntryElement()
        .withIdentifier(dmnDecisionTable.getDecision().getDmnDefinition().getDefinitionId())
        .withIdentifier(dmnDecisionTable.getDecision().getDecisionId())
        .withIdentifier(dmnDecisionTable.getDecisionTableId());
  }

  public VerificationResultEntryElement withIdentifier(AbstractId value) {

    // Check if annotation is present
    Validate.isTrue(
        value.getClass().isAnnotationPresent(JsonIdentifier.class),
        "No annotation \"JsonIdentifier\" for AbstractId %s present.",
        value.getClass().getSimpleName());
    String keyValue = value.getClass().getAnnotation(JsonIdentifier.class).value();
    // every identifier can only be set once
    Validate.isTrue(
        !identifier.containsKey(keyValue),
        "Identifier %s with value %s in %s already set.",
        keyValue,
        value,
        VerificationResultEntryElement.class.getSimpleName());
    identifier.put(keyValue, value);
    return this;
  }
}
