package de.unikoblenz.fgbks.core.dmn.verification.result;

import de.unikoblenz.fgbks.base.builder.DefaultBuilder;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.AbstractVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.VerificationType;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import javax.json.bind.annotation.JsonbProperty;
import org.apache.commons.lang3.Validate;

public class VerifierResult extends AbstractResultObject {

  private VerificationType verificationType;
  private Set<VerificationResultEntry> verificationResultEntries;

  public VerifierResult() {
    this.verificationResultEntries = ConcurrentHashMap.newKeySet();
  }

  @JsonbProperty("type")
  public VerificationType getVerificationType() {
    return verificationType;
  }

  @JsonbProperty("entries")
  public Set<VerificationResultEntry> getVerificationResultEntries() {
    return new HashSet<>(verificationResultEntries);
  }

  @JsonbProperty("size")
  public int getAmountOfEntries() {
    return verificationResultEntries.size();
  }

  public static Builder getBuilder() {
    return new VerifierResult().new Builder();
  }

  public class Builder extends DefaultBuilder<VerifierResult> {

    public Builder withVerificationType(VerificationType verificationType) {
      value.verificationType = Validate.notNull(verificationType);
      return this;
    }

    public Builder addVerificationResultEntry(VerificationResultEntry verificationResultEntry) {
      value.verificationResultEntries.add(Validate.notNull(verificationResultEntry));
      return this;
    }

    public Builder fromVerifier(AbstractVerifier verifier) {
      return withVerificationType(Validate.notNull(verifier).getVerificationType());
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.verificationType);
    }
  }
}
